#!/usr/bin/env perl

# (c) 2002 Ed Green, UC Regents

use strict;
use FamNorm;
use Getopt::Std;
use vars qw( $opt_a $opt_f );

MAIN : {
    
    &init();
    
    my $fno = FamNorm->new( $opt_a );

    my $num_queries = $fno->totalEntries();

    open( FP_FILE, $opt_f ) or die( "$!: $opt_f" );
    
    # Since every line is an error (a false positive), we'll increment
    # this by one for each line 
    my $errors = 0;
    my ( $score, $old_score );

    while ( <FP_FILE> ) {

	$errors++;
	( $score = &processLine( $_ ) );

	unless ( $score == $old_score ) {
	    print ( $score, " ", ($errors / $num_queries), "\n" );
	    $old_score = $score;
	}
    }
}

sub processLine {
    my $line = shift;
    my @el = split( /\s+/, $line );
    return $el[ 2 ];
}

sub init {
    getopts( 'a:f:' );
    unless ( $opt_a && $opt_f ) {
	print( "EPQvScore.pl\n" );
	print( "Generates data showing how many errors were made\n" );
	print( "each significance score level.\n" );
	print( "OPTIONS:\n" );
	print( "-a [ ASTRAL database file for this search ]\n" );
	print( "-f [ File of false positives ]\n" );
	exit( 0 );
    }
}
