# $id $

# FamNormLite.pm

=head1 NAME

    FamNormLite.pm

=cut

=head1 SYNOPSIS

    my $fam_norm_lite_obj = new FamNormLite( 'astral_file' );
    print $fam_norm_lite_obj->totalEntries();

=cut

=head1 DESCRIPTION 

    FamNormLite is a simple module which reads an astral database file and
    stores information about the sequence IDs contained within it.
    Methods are available for extracting this information. The code is 
    identical to Ed Green's FamNorm package developed for running a standard
    bootstrap, except that a large amount of code that is unnecessary (and 
    in some cases would waste memory or cause errors) when using the module
    for a Bayesian bootstrap has been deleted. In particular, the 
    normalization code has been deleted and recoded in BBNorm.pm. The result
    is a small package for parsing Astral databases in preparation for a 
    bootstrap.

=cut

=head1 AUTHOR

    (c) 2002 Ed Green, UC Regents. Gavin Price deleted a bunch of stuff 
    he didn't need. If this is broken, it's his fault.

=cut 

package FamNormLite;
use strict;

=head2 new

  Title:    new
  Function: constructor
  Returns:  a new FamNormLite object
  Args:     an astral database

=cut

sub new {
    my ( $class, $astral_file ) = @_;
    my $self = &_parseInput( $astral_file );
    bless( $self, $class );
    return $self;
}

=head2 astralIds

  Title:    astralIds
  Function: accessor
  Returns:  hash of astral ids for this set
  Args:     none

=cut

sub astralIds {
    my $self = shift;
    my %ids =  %{ $self->{ 'astral_ids' } };
    return %ids;
} 

=head2 sfIds

  Title:    sfIds
  Function: accessor
  Returns:  array of superfamily ids
  Args:     none

=cut

sub sfIds {
    my $self = shift;
    my @ids = keys %{ $self->{ 'SFs' } };
    return @ids;
}

=head2 idToSF

  Title:    idToSF
  Function: accessor
  Returns:  superfamily identifier for a given astral identifier
  Args:     astral identifier( for example, 'd1dlwa_' )

=cut

sub idToSF {
    my $self = shift;
    my $asid = shift;
    return $self->{ 'id2sf' }->{ $asid };
}

=head2 totalEntries

  Title:    totalEntries
  Function: accessor
  Returns:  scalar number of entries in this astral database.
    Same as sum of size of all families.
  Args:     none

=cut

sub totalEntries {
    my $self = shift;
    return $self->{ 'totalEntries' };
}

=head2 _parseInput

  Title:    _parseInput
  Function: internal
  Returns:  object data structre
  Args:     astral filename

=cut

sub _parseInput {
    my $astral_file = shift;


    my ( %self, %id2sf, %astral_ids );
    my ( @elements );
    my ( $l, $id, $sf, $entries );

    # Parse astral database
    open( AST, $astral_file ) or 
	die( "can not parse $astral_file: $!" );
    while( $l = <AST> ) {
	if ( $l =~ /^>/ ) {
	    my @elements = split( /\s+/, $l );
	    ($id) = ($elements[0] =~ /^>(.+)/);
	    ($sf) = ($elements[1] =~ /^(\w+\.\d+\.\d+)\.\d+/);
	    $id2sf{ $id } = $sf;
	    $astral_ids{ $id } += 1; 
            # all entries should be
            # present once when we parse the astral database
	    $entries++;
	}
    }
    close( AST ) or die( "$astral_file: $!" );
    $self{ 'astral_ids' } = \%astral_ids;
    $self{ 'totalEntries' } = $entries;
    $self{ 'id2sf' } = \%id2sf;
    
    return \%self;
}
1;
