#!/usr/bin/env perl

# (c) 2002 Ed Green (original bootstrap implmentation), Gavin Price
# (updated for dbvdb files and added Bayesian bootstrap implementation), UC Regents

use strict;

# Standard bootstrap packages
use FamNorm;

#Bayesian bootstrap packages/routines
use FamNormLite;
use BBNorm;
require 'bayBoot.pl';

use Getopt::Std;
use vars qw( $opt_a $opt_n $opt_f $opt_b $opt_i);

my $DEBUG = 0;
my $dbvdbFlag = 0;

MAIN : {
    
    my $TMP_FN = '/tmp/aBcTmP.txt';

    my ( 
	 $fno, 
	 $fno_target,
	 $db_size,
	 $origFileDB,
	 $i,
	 $i_inner,
	 $astral_id,
	 $j,
	 $sbjt,
	 $iD1,
	 $iD2
	 );
    my ( 
	 @astral_ids 
	 );
    my ( 
	 %SRS_astral 
	 );
    my ($c1, $c2, $c3, $c4, $c5);

    &init();

    if ($opt_i < 0 or $opt_i == undef) { $opt_i = 1 }
        
    # parse Sasum file
    # $sasum_pnt-> { 'query_id' => { 'subj_id1' => [ score, scop ],
    #                                'subj_id2' => [ score, scop ],
    #                                         ...               } }
    # also stores version & db info in keys: Ver, QueryDBsize, TargetDBsize,
    # QueryDB, TargetDB
    $origFileDB = &parseOrigFile( $opt_f );

    if($opt_b) {    #perform Bayesian bootstrap if user specifies (Gavin's code)

	my $fnlo = FamNormLite->new($opt_a);
	my $fnlo_target = $fnlo; #need to add commmand line option & finish coding below to 
	#                            use target DB


	for ($i = $opt_i; $i < $opt_i + $opt_n; $i++) {    #run user-spec'd # of times.
	    my %astral_ids = $fnlo->astralIds();     #get database hash of astral ids
	    my @astral_array = keys %astral_ids;
	    $db_size = $fnlo->totalEntries();
	
	    my @BBgaps = @{&bayBoot($db_size)};	     #make gaps for this database
	    for ($i_inner=0; $i_inner<$db_size; $i_inner++){  #assign gaps to ids
		$astral_ids{$astral_array[$i_inner]} *= $BBgaps[$i_inner];
	    }
	    
	    #BBNorm calculates weights and linear and quadratic normalization factors
	    #for each hit in the astral DB as well as coverage sizes
	    my $bbno = BBNorm->new(\%astral_ids, $fnlo, $origFileDB);  
	    my $bbno_target = $bbno; #yadda yadda yadda you know the drill-
	    
	    open(BBOOT, ">$opt_f.bbs.$i") or die( "$opt_f.bbs.$i: $!");
	    if ($dbvdbFlag) {
		print BBOOT (&makeDbvdbHeader($bbno, $bbno_target, $origFileDB, 1));
 	    } else {
		print BBOOT (&makeSasHeader($bbno, $db_size, 1));
	    }

	    for $iD1 (keys %{$origFileDB}) { #run through each id
		$c1 = $iD1 ne "Ver";         #unless you've got a misc. info. key
		$c2 = $iD1 ne "QueryDB";
		$c3 = $iD1 ne "TargetDB";
		$c4 = $iD1 ne "QueryDBsize";
		$c5 = $iD1 ne "TargetDBsize";
		if ($c1 and $c2 and $c3 and $c4 and $c5) {
		    for $iD2 (keys %{$origFileDB->{$iD1}}) {  #write each hit for that id
			if ($iD1 ne $iD2) {                   #yeah, boyeee
			    print BBOOT (&makeBBLine($iD1, $iD2, $origFileDB, $bbno));
			}
		    }
		}
	    }
	    close (BBOOT);
	}

    }else {    #otherwise, perform standard bootstrap (Ed's code) 

	# make FamNorm Object using astral file
	my $fno = FamNorm->new( $opt_a );
	my $fno_target = $fno;    #need to add command line option & assignent to 
	#                             use target DB

	@astral_ids = $fno->astralIds();
	$db_size    = $fno->totalEntries();

	for ($i = $opt_i; $i < $opt_i + $opt_n; $i++) {    #run user-spec'd # of times.
	    # make SRS w/ replacement hash, %SRS_astral
	    %SRS_astral = ();
	    for ( $j = 1; $j <= $db_size; $j++ ) {
		$SRS_astral{ $astral_ids[ int( rand( $db_size ) ) ] } += 1;
	    }
    
	    # make tmp file with list of ids to restrict to
	    &makeRestrictTmp( $TMP_FN, \%SRS_astral );
	    $fno->restrict( $TMP_FN );
	    open( BOOT, ">$opt_f.bs.$i" ) or die( "$opt_f.bs.$i: $!" );

	    if ($dbvdbFlag) {
		print BOOT ( &makeDbvdbHeader( $fno, $fno_target, $origFileDB ) );
	    } else {
		print BOOT ( &makeSasHeader( $fno, $db_size ) );
	    }
	
	    my $total_lines = 0;

	    # foreach database entry which was sampled
	    foreach $astral_id ( keys %SRS_astral ) {
	    
		# foreach time it was sampled
		for ( $j = 1; $j <= $SRS_astral{ $astral_id }; $j++ ) {
		
		    # get all the sasum lines for this subject
		    foreach $sbjt ( keys %{ $origFileDB->{ $astral_id } } ) {
			if ( $SRS_astral{ $sbjt } ) {
			    for ( my $m = 1; $m <= $SRS_astral{ $sbjt }; $m++ ) {
				print BOOT 
				    ( &makeLine( $astral_id, $sbjt, 
						    $origFileDB, $fno ) );
				$total_lines++;
			    }
			}
		    }
		}
	    }
	    close ( BOOT );

	    my @cov_sizes = $fno->covSizes();

	    if ( ($total_lines != $cov_sizes[ 0 ] ) && ( $DEBUG == 1) ) {
		print ( "PROBLEM\n" );
	    }
	}
	system( "rm $TMP_FN" );
    }
}

sub parseOrigFile {
    my $orig_file = shift;

    my %origDB;
    my ($line1, $line2, $dummyvar);
    my (@elem, @line);

    open (OFILE, $orig_file) or die("$orig_file: $!");
    chomp($line1 = <OFILE>);
    $line2 = <OFILE>;
    
    if ($line1 =~ /sasum/ && $line2 =~/^Expect/) {
	print("Parsing sasum file.\n");
	$dbvdbFlag = 0;
    }elsif ($line1 =~ /^dbvdbsum/ && $line2 =~ /^Query database/) {
	print("Parsing dbvdb file.\n");
	$origDB{ "Ver" } = $line1;                  #get misc file info
	@line = split /\s+/, $line2;
	$origDB{ "QueryDB" } = $line[2];
	@line = split /\s+/, $dummyvar = <OFILE>;
	$origDB{ "TargetDB" } = $line[2];
	@line = split /\s+/, $dummyvar = <OFILE>;
	$origDB{ "QueryDBsize" } = $line[3];
	@line = split /\s+/, $dummyvar = <OFILE>;
	$origDB{ "TargetDBsize" } = $line[3];
	for(0..3) {$dummyvar = <OFILE>}             #skip 4 lines
	$dbvdbFlag = 1;
    } else {
	warn("this does not look like an UNSORTED sasum or dbvdbsum file");
	exit ( 1 );
    }
    
    while ( <OFILE> ) {
	@elem = split( /\s+/, $_ );
	$origDB{ $elem[ 0 ] }->{ $elem[ 1 ] } = [ @elem[ 2 .. 3 ] ];
    }
    close( OFILE );
    return \%origDB;
}

sub checkSasumHeader {                      #no longer used
    unless ( $_[ 0 ] =~ /sasum/ &&
	     $_[ 1 ] =~ /Expect/ ) {
	warn( "this does not look like an UNSORTED sasum file" );
	exit( 1 );
    }
}

sub makeBBLine {
    my $iD1 = shift;
    my $iD2 = shift;
    my $origFileDB = shift;
    my $bbno = shift;
    my $line;

    my @norm_facts = @{$bbno->normFactor($iD1, $iD2)};

    if (!$dbvdbFlag) {
	($norm_facts[1], $norm_facts[2]) = ($norm_facts[2], $norm_facts[1]);
    }
    
    $line = "$iD1 $iD2 ";
    $line .= $origFileDB->{ $iD1 }->{ $iD2 }->[ 0 ];       #E-Value
    $line .= " ";
    $line .= $origFileDB->{ $iD1 }->{ $iD2 }->[ 1 ];       #Scop relation code
    $line .= " ";
    $line .= $norm_facts[ 0 ];                             #Unnorm
    $line .= " ";
    $line .= $norm_facts[ 1 ];                             #Lin Norm
    $line .= " ";
    $line .= $norm_facts[ 2 ];                             #Quad Norm
    $line .= "\n";

    return $line;
}

#Note: the FamNorm package orders the normalization array thusly: (Unorm, Quad Norm, 
#Lin Norm). Hence they have to be switched when writing Dbvdb files. Vice versa for 
#BBNorm and Sasum files.
sub makeLine {
    my $ast_id    = shift;
    my $sbj_id    = shift;
    my $origFileDB = shift;
    my $fno       = shift;
    my $line;

    my @norm_facts = $fno->normFactor( $fno->idToSF( $ast_id ) );

    if ($dbvdbFlag) {
	($norm_facts[1], $norm_facts[2]) = ($norm_facts[2], $norm_facts[1]);
    }

    $line = "$ast_id $sbj_id ";
    $line .= $origFileDB->{ $ast_id }->{ $sbj_id }->[ 0 ];
    $line .= " ";
    $line .= $origFileDB->{ $ast_id }->{ $sbj_id }->[ 1 ];
    $line .= " ";
    $line .= $norm_facts[ 0 ];
    $line .= " ";
    $line .= $norm_facts[ 1 ];
    $line .= " ";
    $line .= $norm_facts[ 2 ];
    $line .= "\n";

    return $line;
}

#same issue as above for Cov arrays
sub makeDbvdbHeader {
    my $fno = shift;
    my $fno_target = shift;
    my %origFileDB = %{ shift(@_) };
    my $bayesian = shift;
    my $head_string;
    my @covSizes = $fno->covSizes();
    my @targetCovSizes = $fno_target->covSizes();

    if ($bayesian) {
	($covSizes[1], $covSizes[2]) = ($covSizes[2], $covSizes[1]);
	($targetCovSizes[1], $targetCovSizes[2]) = ($targetCovSizes[2], $targetCovSizes[1]);
    }
    
    $head_string = "$origFileDB{Ver}\n";
    $head_string .= "Query database: $origFileDB{QueryDB}.bs\n";
    $head_string .= "Target database: $origFileDB{TargetDB}.bs\n";
    $head_string .= "Query database size: $origFileDB{QueryDBsize}\n";
    $head_string .= "Target database size: $origFileDB{TargetDBsize}\n";
    $head_string .= "Query superfamily relations: $covSizes[0] $covSizes[2] $covSizes[1]\n";
    $head_string .= "Target superfamily relations: $targetCovSizes[0] $targetCovSizes[2] $targetCovSizes[1]\n";
    $head_string .= "query_id target_id score scop_rel unnorm lin_norm quad_norm\n";
    $head_string .= "__HITS__\n";
    return $head_string;
}

sub makeSasHeader {
    my $fno = shift;
    my $num_queries = shift;
    my $bayesian = shift;
    my $head_string;
    my @covSizes = $fno->covSizes();

    if ($bayesian) {
	($covSizes[1], $covSizes[2]) = ($covSizes[2], $covSizes[1]);
    }

    $head_string = ( "\$Id: sasum, v 2.1 1998/05/18 06:20:49 brenner Exp brenner \$\n" );
    $head_string .= ( "Expect 0 fa, ".
		      $covSizes[ 0 ]." sf, ".
		      "0 cf, 0 cl\n" );
    $head_string .= ( "Cov Sizes ".
		      $covSizes[ 0 ]." ".
		      $covSizes[ 1 ]." ".
		      $covSizes[ 2 ]."\n" );
    $head_string .= ( "Queries $num_queries\n" );
    return $head_string;
}

# makes a tmp file with ids in it, 1 per line.  Each id shows up the same
# number of times it was sampled
sub makeRestrictTmp {
    my $fn = shift;
    my $SRS_pnt = shift;
    my $id;
    open( TMP, ">$fn" ) or die ( "could not make tmp file: $fn: $!" );
    foreach $id ( keys %{ $SRS_pnt } ) {
	for ( my $i = 1; $i <= $SRS_pnt->{ $id }; $i++ ) {
	    print TMP "$id\n" if $id;
	}
    }
    close( TMP );
}

sub init {
    getopts( 'a:n:f:i:b' );
    unless( $opt_a && $opt_n && $opt_f ) {
	print ( "bootStrap2.pl\n" );
	print( "This program does bootstrapping and family normalization\n" );
	print( "on the output of a database versus database comparison\n" );
	print( "run.  It needs to know the astral database used, the\n" );
	print( "sasum or dbvdb file with the results, and the number of\n" );
	print( "bootstraps to do.\n" );
	print( "OPTIONS:\n" );
	print( "-a [ astral database filename ]\n" );
	print( "-n [ number of bootstrap iterations ]\n" );
	print( "-f [ sasum or dbvdb file ]\n" );
	print( "-i [ starting number for bootstrap filename id number ]\n");
	print( "   if number < 0, number will be set to 1\n");
	print( "-b : perform Bayesian bootstrap\n" );
	exit( 0 );
    }
}
