#!/usr/bin/env perl

# (c) 2002 Ed Green, UC Regents
# 2nd part of implementation of 'The bootstrap algorithm for
# estimating standard errors' - Algorithm 6.1 - p.47 of _An
# Introduction to the Bootstrap_, Efron and Tibshirani


use Getopt::Std;
use strict;

use vars qw( $opt_e $opt_i $opt_g $opt_r );

my $COV_BIN_SIZE = 0.005;

MAIN : {
    my ( @cov );
    my ( $mu, $std );

    &init();
   
    # get array of coverages at specified EPQ level
    @cov = &readGnuFiles();
    
    # do some stats
    ( $mu, $std ) = &getStats( @cov );
    
    # generate output
    &output( $mu, $std, @cov );

}

sub getStats {
    my ( $sum, $datum, $avg, $foo, $std );
    my @data = @_;


    # find avg
    foreach $datum ( @data ) {
	$sum += $datum;
    }
    $avg = $sum / ($#data + 1);

    # find std
    foreach $datum ( @data ) {
	$foo += (( $datum - $avg ) * ( $datum - $avg )) / $#data;
    }
    $std = sqrt( $foo );
    return ( $avg, $std );
}    

sub output {
    my $mu  = shift;
    my $std = shift;
    my $e_string = ( "Expected value for coverage at $opt_e EPQ is  $mu\n" );
    my $s_string = ( "Stdandard error for coverage at $opt_e EPQ is $std\n" );
    
    print( "$e_string\n$s_string\n" );

    if ( $opt_g ) {
	open( GNUOUT, ">$opt_g.dat" ) or die( "$opt_g.dat: $!" );
	print GNUOUT ( "# $opt_g data in $opt_i at $opt_e\n" );
	print GNUOUT ( "# $e_string\n" );
	print GNUOUT ( "# $s_string\n" );
	my $i;
	my $cov;
	my @cov_bin;
	
	foreach $cov ( @_ ) {
	    $cov_bin[ int( $cov / $COV_BIN_SIZE ) ]++;
	}

	for ( $i = 0; $i <= $#cov_bin; $i++ ) {
	    print GNUOUT ( $i * $COV_BIN_SIZE );
	    print GNUOUT ( " " );
	    unless ( $cov_bin[ $i ] ) {
		print GNUOUT ( " 0\n" );
	    }
	    else {
		print GNUOUT ( $cov_bin[ $i ] );
		print GNUOUT ( "\n" );
	    }
	}

	print GNUOUT ( $i * $COV_BIN_SIZE );
	print GNUOUT ( " 0\n" );

	close( GNUOUT );
	
	open ( GS, ">$opt_g.gs" ) or die ( "$opt_g.gs: $!" );
	my $var = $std * $std;
	print GS ( "# Gnuplot script file for data in $opt_i at $opt_e\n" );
	print GS ( "set terminal postscript color;\n" );
	print GS ( "set grid;\n" );
	print GS ( "set output \"$opt_g.ps\";\n" );
	print GS ( "plot [0:1.00][0:40] \"$opt_g.dat\" with histeps,\\\n" );
	print GS ( "1/($std*sqrt(2*3.14)*exp(((x-$mu)*(x-$mu))/(2*$var)))\n" );
	close ( GS );
    }

}

sub readGnuFiles {
    my $input;
    my @cov;
    opendir( DIR, $opt_i ) or die( "$opt_i: $!" );
    my @inputs = readdir( DIR );
    closedir( DIR );
    
    foreach $input ( @inputs ) {
	if ( $input ne '.' && $input ne '..' &&
	     index( $input, $opt_r ) != -1 ) {
	    push( @cov, &getCov( "$opt_i$input" ) );
	}
    }
    return @cov;
}

sub getCov {
    my $fn = shift;
    my ( $r, $last_cov );
    my ( @dp );
    
    
    if ( $fn =~ /\.gz?/ ) {
	open( GNU, "gzcat $fn|" ) or die( "$fn: $!" );
    }

    elsif ( $fn =~ /\.bz2?/ ) {
	open( GNU, "bzcat $fn|" ) or die( "$fn: $!" );

    }

    else {
	open( GNU, "$fn" ) or die( "$fn : $!" );
    }
    
    while( $r = <GNU> ) {
	unless ( $r =~ /\#/ ) {
	    @dp = split( /\s+/, $r );
	    if ( $dp[ 1 ] >= $opt_e ) {
		close( GNU );
		return $last_cov;
	    }
	    else {
		$last_cov = $dp[ 0 ];
	    }
	}
    }
    close(GNU);
    return $last_cov;
}

sub endsIn {
    my $char = shift;
    my $string = shift;
    my $len = length( $string );
    if ( substr( $string, ($len - 1), 1  ) eq $char ) {
	return 1;
    }
    else {
	return 0;
    }
}

sub init {
    getopts( 'i:e:g:r:' );
    unless ( $opt_i && $opt_e ) {
	print( "covDist.pl - Report stats on the coverage distribution\n" );
	print( "at a user specified Errors Per Query (EPQ) rate for a\n" );
	print( "set of inputs.\n" );
	print( "OPTIONS:\n" );
	print( "-i [ input directory with gnuplot style data ]\n" );
	print( "-e [ EPQ cutoff ]\n" );
	print( "-g [ filename for gnuplot data file of distribution; if ]\n" );
	print( "     not specified, none is made ]\n" );
	print( "-r [ filename restriction for gnuplot datafile ]\n" );
	exit( 0 );
    }
    unless ( &endsIn( '/', $opt_i ) ) {
	$opt_i .= '/';
    }
}


