#!/usr/bin/env perl

# (c) 2002 Ed Green, UC Regents

use strict;
use Getopt::Std;
use DBVDB;
use vars qw( $opt_i $opt_o $opt_p $opt_v );

MAIN : {

    my ( $dbo, $sig, $hit, $hit_p, $epq, 
	 $qdb_size, $tdb_size, $errors );

    my ( @cve, $query_rels, $target_rels, @cov );

    &init();
    
    $errors = 0;
    
    $dbo = new DBVDB( $opt_i );
    $qdb_size = $dbo->numQueries();
    $tdb_size = $dbo->numTargets();
    $query_rels = $dbo->numQueryRels();
    $target_rels = $dbo->numTargetRels();

    foreach $sig ( $dbo->sigs() ) {
	$hit_p = $dbo->hitsAtSig( $sig );
	$errors = 
	    &examineHits( $hit_p, $errors, \@cov, $query_rels,
			  $target_rels );
	&updateCVE( $errors, $qdb_size, \@cov, \@cve );
    }
    &makeDataFile( \@cve ); 
}

sub examineHits {
    my $hit_p = shift;
    my $errors = shift;
    my $cov_p = shift;
    my $qr_p = shift;
    my $tr_p = shift;;
    my $rel;
    my ( $hit );
    foreach $hit ( @{ $hit_p } ) {
	$rel = &related( $hit );
	if ( $rel == 1 ) {
	    &addToCov( $cov_p, $hit, $qr_p, $tr_p );
	}
	elsif ( $rel == -1 ) {
	    $errors++;
	    if ( $opt_v ) {
		&showError( $hit );
	    }
	}
	else {
	    # undecided, rel == 0, do nothing
	}
    }
    return $errors;
}

sub related {
    my $hit = shift;
    if ( $hit->[ 3 ] == 3 ) {
	return 1;
    }
    elsif ( $hit->[ 3 ] == 2 ) {
	return 0;
    }
    else {
	return -1;
    }
}

sub addToCov {
    my $cov_p = shift;
    my $hit = shift;
    my $qr_p = shift;
    my $tr_p = shift;
    
    if ( $opt_p ) {
	# Profile method, only do linear normalization
	$cov_p->[ 0 ] += $hit->[ 4 ] / $tr_p->[ 0 ];
	$cov_p->[ 1 ] += $hit->[ 5 ] / $tr_p->[ 1 ];
    }
    
    else {
	# Query database = target database
	$cov_p->[ 0 ] += $hit->[ 4 ] / $qr_p->[ 0 ];
	$cov_p->[ 1 ] += $hit->[ 5 ] / $qr_p->[ 1 ];
	$cov_p->[ 2 ] += $hit->[ 6 ] / $qr_p->[ 2 ];
    }
}

sub showError {
    my $hit = shift;
    my $qid = $hit->[ 0 ];
    my $tid = $hit->[ 1 ];
    my $sig = $hit->[ 2 ];
    my $rel = $hit->[ 3 ];

    print( "$qid $tid $sig $rel\n" );
}

sub updateCVE {
    my $errors = shift;
    my $qdb_size = shift;
    my $cov_p = shift;
    my $cve_p = shift;
    push( @{$cve_p->[ 0 ]}, [ $cov_p->[ 0 ], $errors/$qdb_size ] );
    push( @{$cve_p->[ 1 ]}, [ $cov_p->[ 1 ], $errors/$qdb_size ] );
    push( @{$cve_p->[ 2 ]}, [ $cov_p->[ 2 ], $errors/$qdb_size ] );
}

sub makeDataFile {
    my $cve_p = shift;
    my ( $point, 
	 $cov, $epq, 
	 $cov_m1, $epq_m1,
	 $cov_m2, $epq_m2 ) = 
	     ( 0, 0, 0, 0, 0, 0, 0 );
    open( OUTFILE, ">$opt_o" ) or die( "$opt_o: $!" );
    print OUTFILE ( "# Unnormalized CVE data\n" );
    foreach $point ( @{ $cve_p->[ 0 ] } ) {
	( $cov, $epq, $cov_m1, $epq_m1, $cov_m2, $epq_m2 ) = 
	    ( $point->[ 0 ], $point->[ 1 ],
	      $cov, $epq, $cov_m1, $epq_m1 );
	if ( &showPoint( $cov, $epq, $cov_m1, $epq_m1, 
			 $cov_m2, $epq_m2 ) ) {
	    print OUTFILE ("$cov_m1 $epq_m1\n" );
	}
    }
    print OUTFILE ( "\n\n" );
    
    ( $point, $cov, $epq, $cov_m1, $epq_m1, $cov_m2, $epq_m2 ) = 
	( 0, 0, 0, 0, 0, 0, 0 );
    print OUTFILE ( "# Linearly normalized CVE data\n" );
    foreach $point ( @{ $cve_p->[ 1 ] } ) {
	( $cov, $epq, $cov_m1, $epq_m1, $cov_m2, $epq_m2 ) = 
	    ( $point->[ 0 ], $point->[ 1 ],
	      $cov, $epq, $cov_m1, $epq_m1 );
	if ( &showPoint( $cov, $epq, $cov_m1, $epq_m1, 
			 $cov_m2, $epq_m2 ) ) {
	    print OUTFILE ("$cov_m1 $epq_m1\n" );
	}
    }
    print OUTFILE ( "\n\n" );
    
    ( $point, $cov, $epq, $cov_m1, $epq_m1, $cov_m2, $epq_m2 ) = 
	( 0, 0, 0, 0, 0, 0, 0 );
    print OUTFILE ( "# Quadratically normalized CVE data\n" );
    foreach $point ( @{ $cve_p->[ 2 ] } ) {
	( $cov, $epq, $cov_m1, $epq_m1, $cov_m2, $epq_m2 ) = 
	    ( $point->[ 0 ], $point->[ 1 ],
	      $cov, $epq, $cov_m1, $epq_m1 );
	if ( &showPoint( $cov, $epq, $cov_m1, $epq_m1, 
			 $cov_m2, $epq_m2 ) ) {
	    print OUTFILE ("$cov_m1 $epq_m1\n" );
	}
    }
    close OUTFILE
}

sub showPoint {
    my $cov = shift;
    my $epq = shift;
    my $cm1 = shift;
    my $em1 = shift;
    my $cm2 = shift;
    my $em2 = shift;
    
    return 1;

    unless ( ($cov == $cm1 && $cm1 == $cm2) ||
	     ($epq == $em1 && $em1 == $em2) ) {
	return 1;
    }
    else {
	return 0;
    }
}

sub init {
    getopts( 'i:p:v:o:' );
    unless ( $opt_i ) {
	print( "cve_maker.pl\n" );
	print( "Examines a dbvdb file and outputs CVE data\n" );
	print( "OPTIONS:\n" );
	print( "-i [ Input dbvdb output file ]\n" );
	print( "-o [ Output CVE file ]\n" );
	print( "-p [ Profile method results ]\n" );
	print( "-v [ verbose mode; report false positives ]\n" );
	exit( 0 );
    }
    unless ( $opt_o ) {
	$opt_o = "$opt_i.dat";
    }
}
