#!/usr/bin/perl

use strict;
use Getopt::Std;
use vars qw( $opt_i $opt_e $opt_r );

MAIN : {

    my ( %un_norm, %lin_norm, %quad_norm );
    my ( @gnufiles );
    my ( $gnufile, $found_un, $found_lin, $found_quad, 
	 $r, $cov, $epq );

    &init();

    opendir( GNU, $opt_i ) or die( "$opt_i: $!" );
    @gnufiles = readdir( GNU );
    closedir( GNU );

    foreach $gnufile ( @gnufiles ) {
        if ( &hasRest( $gnufile ) ) {
	    open( GNU, "$opt_i$gnufile" ) or die( "$gnufile: $!" );

	    ( $found_un, $found_lin, $found_quad ) = ( 0, 0, 0 );
	    until ( $found_un ) {
		$r = <GNU>;
		unless ( $r =~ /^\#/ ) {
		    ( $cov, $epq ) = split( /\s+/, $r );
		    if ( $epq > $opt_e ) {
			$un_norm{ $gnufile } = $cov;
			$found_un = 1;
		    }
		}
	    }
	    
	    # Go to linearly normalized section
	    $r = <GNU>;
	    until ( $r =~ /^\# Linearly normalized/ ) {
		$r = <GNU>;
	    }
	    
	    until ( $found_lin ) {
		$r = <GNU>;
		( $cov, $epq ) = split( /\s+/, $r );
		if ( $epq > $opt_e ) {
		    $lin_norm{ $gnufile } = $cov;
		    $found_lin = 1;
		}
	    }

	    # Go to quadratically normalized section
	    $r = <GNU>;
	    until ( $r =~ /^\# Quadratically normalized/ ) {
		$r = <GNU>;
	    }
	    
	    until ( $found_quad ) {
		$r = <GNU>;
		( $cov, $epq ) = split( /\s+/, $r );
		if ( $epq > $opt_e ) {
		    $quad_norm{ $gnufile } = $cov;
		    $found_quad = 1;
		}
	    }

	    close( GNU );
				
	}
    }

    print( "Unnormalized:\n" );
    foreach $gnufile ( sort {$un_norm{$b} <=> $un_norm{$a} } keys %un_norm ) {
	print( "$gnufile => ", $un_norm{ $gnufile }, "\n" );
    }

    print( "Linearly normalized:\n" );
    foreach $gnufile ( sort {$lin_norm{$b} <=> $lin_norm{$a} } keys %lin_norm ) {
	print( "$gnufile => ", $lin_norm{ $gnufile }, "\n" );
    }

    print( "Quadratically normalized:\n" );
    foreach $gnufile ( sort {$quad_norm{$b} <=> $quad_norm{$a} } keys %quad_norm ) {
	print( "$gnufile => ", $quad_norm{ $gnufile }, "\n" );
    }

}

sub hasRest {
    my $gnufile = shift;
    if ( index( $gnufile, $opt_r ) != -1 ) {
	return 1;
    }
    else {
	return 0;
    }
}

sub endsIn {
    my $char = shift;
    my $string = shift;
    my $len = length( $string );
    if ( substr( $string, ($len - 1), 1  ) eq $char ) {
	return 1;
    }
    else {
	return 0;
    }
}

sub init {
    getopts( 'i:e:r:' );
    unless ( $opt_i ) {
	print( "maxCov.pl\n" );
	print( "Orders the coverage given at a user defined EPQ\n" );
	print( "from a list of gnu datafiles.  Used to determine\n" );
	print( "the best parameter set.\n" );
	print( "OPTIONS:\n" );
	print( "-i [ Input directory with gnuplot datafiles in\n" );
	print( "     EPQ_maker.pl format.\n" );
	print( "-e [ EPQ level to examine; default = 0.01 ]\n" );
	print( "-r [ filename restriction for data files ]\n" );
	exit( 0 );
    }
    unless ( &endsIn( '/', $opt_i ) ) {
	$opt_i .= '/';
    }

    unless ( $opt_e ) {
	$opt_e = 0.01;
    }
}
    
