#!/usr/bin/env perl

# (c) 2002 Ed Green, UC Regents
# pOverlap.pl give a p-value for the overlap of two bootstrapped
# coverage distributions generated by bootStrap.pl


use Getopt::Std;
use strict;

use vars qw( $opt_o $opt_t $opt_n $opt_e $opt_r );

MAIN : {
    
    my ( @dist1, @dist2 );
    my ( $size1, $size2, $rand1, $rand2, 
	 $num_greater, $i );

    &init();

    # get @dist1 and @dist2 => arrays of the bootstrapped coverages at
    # the given EPQ rate from each distribution
    @dist1 = &getDist( $opt_o );
    @dist2 = &getDist( $opt_t );

    $size1 = $#dist1;
    $size2 = $#dist2;

    for ( $i = 1; $i <= $opt_n; $i++ ) {
	$rand1 = int( rand( $size1 + 1 ) );
	$rand2 = int( rand( $size2 + 1 ) );

	if ( $dist1[ $rand1 ] >= $dist2[ $rand2 ] ) {
	    $num_greater++;
	}
    }

    print( ( $num_greater / $opt_n ), "\n" );
}

sub getDist {
    my $data_dir = shift;
    my $input;
    my @cov;

    unless ( &endsIn( '/', $data_dir ) ) {
	$data_dir .= '/';
    }

    opendir( DIR, $data_dir ) or die( "$data_dir: $!" );
    my @inputs = readdir( DIR );
    closedir( DIR );

    foreach $input ( @inputs ) {
	if ( $input ne '.' && $input ne '..' &&
	     index( $input, $opt_r ) != -1 ) {
	    push( @cov, &getCov( "$data_dir$input" ) );
	}
    }
    return @cov;
}

sub getCov {
    my $fn = shift;
    my ( $r, $last_cov );
    my ( @dp );
    
    
    if ( $fn =~ /\.gz?/ ) {
	open( GNU, "gzcat $fn|" ) or die( "$fn: $!" );
    }

    elsif ( $fn =~ /\.bz2?/ ) {
	open( GNU, "bzcat $fn|" ) or die( "$fn: $!" );

    }

    else {
	open( GNU, "$fn" ) or die( "$fn : $!" );
    }
    
    while( $r = <GNU> ) {
	unless ( $r =~ /\#/ ) {
	    @dp = split( /\s+/, $r );
	    if ( $dp[ 1 ] >= $opt_e ) {
		close( GNU );
		return $last_cov;
	    }
	    else {
		$last_cov = $dp[ 0 ];
	    }
	}
    }
    close(GNU);
    return $last_cov;
}

sub endsIn {
    my $char = shift;
    my $string = shift;
    my $len = length( $string );
    if ( substr( $string, ($len - 1), 1  ) eq $char ) {
	return 1;
    }
    else {
	return 0;
    }
}

sub init {
    getopts( 'o:t:n:e:r:' );
    unless ( $opt_o && $opt_t && $opt_n && $opt_e && $opt_r ) {
	print( "pOverlap.pl - Determines the percent of overlap\n" );
	print( "between two bootstraped coverage distributions\n" );
	print( "by randomly sampling the coverage at a specified\n" );
	print( "EPQ level and counting the percentage of time\n" );
	print( "that the first distribution's coverage is\n" );
	print( "greater than or equal to the second's.\n" );
	print( "OPTIONS:\n" );
	print( "-o [ Input directory 1 ]\n" );
	print( "-t [ Input directory 2 ]\n" );
	print( "-n [ number of times to sample ]\n" );
	print( "-e [ EPQ level to examine ]\n" );
	print( "-r [ filename restriction for input data ]\n" );
	exit( 0 );
    }
}

