#!/usr/bin/env perl

# (c) 2002 Ed Green, UC Regents

# sfSizeDist.pl
# Takes an astral database and uses the FamNorm perl module to
# generate a distribution of the sizes of its superfamilies.  The
# output file looks like this:
#
# # Superfamily Size Distribution of [INPUT ASTRAL DB]
# # BIGGEST_SF_ID         SIZE
# # NEXT_BIGGEST_SF_ID    SIZE
# # .
# # .
# # . 
# # SMALLEST_SF_ID        SIZE
# #
# # SIZE DISTRIBUTION
# 1 [NUM OF SUPERFAMILIES OF SIZE 1]
# 2 [NUM OF SUPERFAMILIES OF SIZE 2]
# 3 [NUM OF SUPERFAMILIES OF SIZE 3]
# .
# .
# .
# X [NUM OF SUPERFAMILIES OF SIZE X]

use strict;
use FamNorm;
use Getopt::Std;
use vars qw( $opt_a );

MAIN : {

    my ( $fno, $sf, $size );
    my ( @sfs, @sizes );

    &init();

    $fno = FamNorm->new( $opt_a );
    
    @sfs = $fno->sfIds();
    
    foreach $sf ( @sfs ) {
	$size = $fno->numSFUniq( $sf );
	push( @{ $sizes[ $size ] }, $sf );
    }
    
    &output( @sizes );

}

sub output {
    my @sizes = @_;
    my ( $sf_pnt, $size, $sf, $num );

    print ( "# Superfamily Size Distribution of $opt_a\n" );
   
    $size = 0;

    for ( $size = 0; $size <= $#sizes; $size++ ) {
	$sf_pnt = $sizes[ $size ];
	foreach $sf ( @{ $sf_pnt } ) {
	    print ( "# $sf $size\n" );
	}
    }

    print( "#\n" );
    print( "# SIZE DISTRIBUTION\n" );
    
    for ( $size = 0; $size <= $#sizes; $size++ ) {
	$sf_pnt = $sizes[ $size ];
	if ( $sf_pnt ) {
	    $num = @{ $sf_pnt };
	}
	else {
	    $num = 0;
	}
	print ( "$size $num\n" );
    }
}

sub init {
    getopts( 'a:' );
    unless ( $opt_a ) {
	print( "sfSizeDist.pl\n" );
	print( "Reports the distribution of superfamily sizes\n" );
	print( "within an astral database.\n" );
	print( "OPTIONS\n" );
	print( "-a [ input astral database ]\n" );
	exit( 0 );
    }
}
